[PATCH 2/8] trace-cmd: Create API tracecmd_read_pre_headers()

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

 



From: "Steven Rostedt (VMware)" <rostedt@xxxxxxxxxxx>

The trace-cmd restore operation can create a partial header to read the
trace event formats and kallsyms and other data into a stand alone header
before it has access to the cpu data. Then it will also read this header to
put together a broken trace, and it reads the header that does not have the
cpu data attached to it.

In order to handle this case, it needs a way to read the headers but stop
short of reading the CPU information. That requires breaking up
tracecmd_read_headers() with just stopping short of adding the cpu data.

A new API is added called tracecmd_read_pre_headers() that does exactly that.

Signed-off-by: Steven Rostedt (VMware) <rostedt@xxxxxxxxxxx>
---
 .../include/private/trace-cmd-private.h       |  1 +
 lib/trace-cmd/trace-input.c                   | 25 ++++++++++++++++---
 tracecmd/trace-restore.c                      |  2 +-
 3 files changed, 24 insertions(+), 4 deletions(-)

diff --git a/lib/trace-cmd/include/private/trace-cmd-private.h b/lib/trace-cmd/include/private/trace-cmd-private.h
index fc968cc9efe1..c7ef3af7c8f7 100644
--- a/lib/trace-cmd/include/private/trace-cmd-private.h
+++ b/lib/trace-cmd/include/private/trace-cmd-private.h
@@ -153,6 +153,7 @@ typedef void (*tracecmd_handle_init_func)(struct tracecmd_input *handle,
 struct tracecmd_input *tracecmd_alloc(const char *file, int flags);
 struct tracecmd_input *tracecmd_alloc_fd(int fd, int flags);
 void tracecmd_ref(struct tracecmd_input *handle);
+int tracecmd_read_pre_headers(struct tracecmd_input *handle);
 int tracecmd_read_headers(struct tracecmd_input *handle);
 int tracecmd_get_parsing_failures(struct tracecmd_input *handle);
 int tracecmd_long_size(struct tracecmd_input *handle);
diff --git a/tracecmd/trace-restore.c b/tracecmd/trace-restore.c
index 13f803053582..bf6940991178 100644
--- a/tracecmd/trace-restore.c
+++ b/tracecmd/trace-restore.c
@@ -122,7 +122,7 @@ void trace_restore (int argc, char **argv)
 		if (!ihandle)
 			die("error reading file %s", input);
 		/* make sure headers are ok */
-		if (tracecmd_read_headers(ihandle) < 0)
+		if (tracecmd_read_pre_headers(ihandle) < 0)
 			die("error reading file %s headers", input);
 
 		handle = tracecmd_copy(ihandle, output);
diff --git a/lib/trace-cmd/trace-input.c b/lib/trace-cmd/trace-input.c
index 9ef7b9f16951..9e1a44540201 100644
--- a/lib/trace-cmd/trace-input.c
+++ b/lib/trace-cmd/trace-input.c
@@ -772,14 +772,17 @@ static int read_cpus(struct tracecmd_input *handle)
 }
 
 /**
  * tracecmd_read_headers - read the header information from trace.dat
  * @handle: input handle for the trace.dat file
  *
  * This reads the trace.dat file for various information. Like the
  * format of the ring buffer, event formats, ftrace formats, kallsyms
- * and printk.
+ * and printk, but stops before reading cpu and options.
+ *
+ * This is needed by the restore operation where the header does not
+ * have the CPU information yet.
  */
-int tracecmd_read_headers(struct tracecmd_input *handle)
+int tracecmd_read_pre_headers(struct tracecmd_input *handle)
 {
 	int ret;
 
@@ -815,6 +818,22 @@ int tracecmd_read_headers(struct tracecmd_input *handle)
 		return -1;
 	handle->file_state = TRACECMD_FILE_CMD_LINES;
 
+	return 0;
+}
+
+/**
+ * tracecmd_read_headers - read the header information from trace.dat
+ * @handle: input handle for the trace.dat file
+ *
+ * This reads the trace.dat file for various information. Like the
+ * format of the ring buffer, event formats, ftrace formats, kallsyms
+ * and printk.
+ */
+int tracecmd_read_headers(struct tracecmd_input *handle)
+{
+	if (tracecmd_read_pre_headers(handle))
+		return -1;
+
 	if (read_cpus(handle) < 0)
 		return -1;
 	handle->file_state = TRACECMD_FILE_CPU_COUNT;
@@ -772,14 +772,17 @@ static int read_cpus(struct tracecmd_input *handle)
 
 /**
- * tracecmd_read_headers - read the header information from trace.dat
+ * tracecmd_read_pre_headers - read the header information from trace.dat
  * @handle: input handle for the trace.dat file
  *
-- 
2.30.0





[Index of Archives]     [Linux USB Development]     [Linux USB Development]     [Linux Audio Users]     [Yosemite Hiking]     [Linux Kernel]     [Linux SCSI]

  Powered by Linux